
#include "chart_acs_spectra.h"
#include "fe/textdrawitem2d.h"
#include "fe/textchartnumbergen2d.h"
#include "wl/sensor_acs.h"



Chart_ACS_Spectra::Chart_ACS_Spectra(QWidget* p) : 
  BufferWidget(p) {

  chart.setBackgroundColour(JColour::White);
  chart.setTickLength(4);
  chart.setMargins(5,15,12,5);
  chart.setNumbersMargin(4);
  chart.setTitleItem(new TextDrawItem2D("ACS"));
 
  TextDrawItem2D* ti = new TextDrawItem2D("a or c (m-1)");
  ti->setRotation(270);
  chart.yLeftAxis().setLabelItem(ti);
  chart.xBottomAxis().setLabelItem(new TextDrawItem2D("Wavelength (nm)"));

  chart.yLeftAxis().setNumberGen(new TextChartNumberGen2D);
  chart.xBottomAxis().setNumberGen(new TextChartNumberGen2D);

  chart.yLeftAxis().setDisplayNumbers(true);
  chart.yLeftAxis().setValueRange(0, 6);
  chart.yLeftAxis().setAxisNumberFormat("%.1f");
  chart.yLeftAxis().setTickSubdivision(0.2, 10);
  chart.yRightAxis().setTickLength(0);
  chart.xBottomAxis().setDisplayNumbers(true);
  chart.xBottomAxis().setValueRange(400, 750);
  chart.xBottomAxis().setAxisDisplayMin(400);
  chart.xBottomAxis().setAxisDisplayMax(750);
  chart.xTopAxis().setTickLength(0);

  a_line.setColour(JColour::Red);
  c_line.setColour(JColour::Blue);

  chart.registerData0(a_line);
  chart.registerData0(c_line);

  ChartKey2D* key = new ChartKey2D();
  key->addItem(new TextDrawItem2D("a"),JColour::Blue);
  key->addItem(new TextDrawItem2D("c"),JColour::Red);
  chart.setKey(key);
}



// ChartACS::sizePolicy()

QSizePolicy Chart_ACS_Spectra::sizePolicy() const {
  return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}



// SpecProj::resizeEvent()
 
void Chart_ACS_Spectra::resizeEvent(QResizeEvent* e) { 
  BufferWidget::resize(width()-2, height()-2);
  drawToBuffer();
}




void Chart_ACS_Spectra::newRecordList(const RecordList* rec_list) {

  // set maximum y-scale

  double y_max = -1;

  JListIterator<Record> iter(*rec_list);
  Record* rec = iter.first();
  while (rec) {

    const Record_ACS* rec_acs = dynamic_cast<const Record_ACS*>(rec);    
    if (rec_acs) {
      double y = norm_max(rec_acs->cCorrected());
      if (y > y_max) y_max = y;
      y = norm_max(rec_acs->aCorrected());
      if (y > y_max) y_max = y;
    }

    rec = iter.next();
  }

  if (y_max < 0) y_max = 6;
  else {
    y_max = floor(y_max * 1.3);
  }
  chart.yLeftAxis().setValueRange(0, y_max);

  drawToBuffer();
  update();
}
 


// Chart_ACS_Spectra::setRecord()

void Chart_ACS_Spectra::newRecordSelected(const Record* rec) {

  a_line.clearData();
  c_line.clearData();
  chart.deleteLabelItems();

  const Record_ACS* rec_acs = dynamic_cast<const Record_ACS*>(rec);

  if (!rec_acs) return;

  const Sensor_ACS* sens_acs = dynamic_cast<const Sensor_ACS*>(rec_acs->sensor());

  if (!sens_acs) return;

  printf("new data\n");

  if (!sens_acs->hasDeviceFile()) {
    chart.addLabelItem(new ChartLabel2D(new TextDrawItem2D("No Device File")));
  } else {
    a_line.setData(sens_acs->aBandWavelength(), rec_acs->aCorrected());
    c_line.setData(sens_acs->cBandWavelength(), rec_acs->cCorrected());
  } 

  drawToBuffer();
  update();
}



void Chart_ACS_Spectra::drawToBuffer() {
  chart.arrangeDrawing(buffer().width(), buffer().height());
  chart.drawToBuffer(buffer());
}

