
#include "chart_ecobb3_time.h"
#include "fe/textdrawitem2d.h"
#include "fe/textchartnumbergen2d.h"
#include "wl/sensor_ecobb3.h"



Chart_ECOBB3_Time::Chart_ECOBB3_Time(QWidget* p) : 
  BufferWidget(p) {
  
  chart.setBackgroundColour(JColour::White);
  chart.setTickLength(4);
  chart.setMargins(5,15,12,5);
  chart.setNumbersMargin(4);
  chart.setTitleItem(new TextDrawItem2D("ECOBB3"));
 
  TextDrawItem2D* ti = new TextDrawItem2D("Counts x 1000");
  ti->setRotation(270);
  chart.yLeftAxis().setLabelItem(ti);
  chart.xBottomAxis().setLabelItem(new TextDrawItem2D("Time (s)"));

  chart.yLeftAxis().setNumberGen(new TextChartNumberGen2D);
  chart.xBottomAxis().setNumberGen(new TextChartNumberGen2D);

  chart.yLeftAxis().setDisplayNumbers(true);
  chart.yLeftAxis().setValueRange(0, 2);
  chart.yLeftAxis().setAxisNumberFormat("%.1f");
  chart.yLeftAxis().setTickSubdivision(0.2, 10);
  chart.yRightAxis().setTickLength(0);
  chart.xBottomAxis().setDisplayNumbers(true);
  chart.xBottomAxis().setValueRange(0, 300);
  chart.xBottomAxis().setAxisDisplayMin(0);
  //chart.xBottomAxis().setAxisDisplayMax(300);
  chart.xTopAxis().setTickLength(0);

  line_470.setColour(JColour::Blue);
  line_530.setColour(JColour::Green);
  line_650.setColour(JColour::Red);

  chart.registerData0(line_470);
  chart.registerData0(line_530);
  chart.registerData0(line_650);

  time_line.setColour(JColour(128,128,128));
  chart.registerData0(time_line);

  ChartKey2D* key = new ChartKey2D();
  key->addItem(new TextDrawItem2D("470 nm"),JColour::Blue);
  key->addItem(new TextDrawItem2D("530 nm"),JColour::Green);
  key->addItem(new TextDrawItem2D("650 nm"),JColour::Red);
  chart.setKey(key);
}



// Chart_ECOBB3::sizePolicy()

QSizePolicy Chart_ECOBB3_Time::sizePolicy() const {
  return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}



// Chart_ECOBB3j::resizeEvent()
 
void Chart_ECOBB3_Time::resizeEvent(QResizeEvent* e) { 
  BufferWidget::resize(width()-2, height()-2);
  drawToBuffer();
}



void Chart_ECOBB3_Time::newRecordList(const RecordList* rec_list) {

  line_470.clearData();
  line_530.clearData();
  line_650.clearData();

  int rec_count = 0;

  JListIterator<Record> iter(*rec_list);
  Record* rec = iter.first();
  while (rec) {

    const Record_ECOBB3* rec_ecobb3 = dynamic_cast<const Record_ECOBB3*>(rec);    
    if (rec_ecobb3) rec_count++;

    rec = iter.next();
  }

  if (!rec_count) return;

  vector<double> x_data(rec_count);
  vector<double> y470_data(rec_count);
  vector<double> y530_data(rec_count);
  vector<double> y650_data(rec_count);

  rec_count = 0;
  
  rec = iter.first();
  while (rec) {

    const Record_ECOBB3* rec_ecobb3 = dynamic_cast<const Record_ECOBB3*>(rec);    
    if (rec_ecobb3) {

      x_data(rec_count) = rec->timeSinceStartMsec() / 1000.0;
      y470_data(rec_count) = rec_ecobb3->sig470() / 1000.0;
      y530_data(rec_count) = rec_ecobb3->sig530() / 1000.0;
      y650_data(rec_count) = rec_ecobb3->sig650() / 1000.0;
      rec_count++;
    }
    rec = iter.next();
  }

  chart.xBottomAxis().setValueRange(0, x_data(rec_count-1));

  line_470.setData(x_data, y470_data);
  line_530.setData(x_data, y530_data);
  line_650.setData(x_data, y650_data);
  
  drawToBuffer();
  update();
}


 
// Chart_ECOBB3::setRecord()

void Chart_ECOBB3_Time::newRecordSelected(const Record* rec) {

  const Record_ECOBB3* rec_ecobb3 = dynamic_cast<const Record_ECOBB3*>(rec);

  if (!rec_ecobb3) return;

  time_line.clearData();

  line_470.setHighlightedPoint(rec->recordPosition());
  line_530.setHighlightedPoint(rec->recordPosition());
  line_650.setHighlightedPoint(rec->recordPosition());

  double x = rec->timeSinceStartMsec() / 1000.0;

  vector<double> x_data_line(2);
  vector<double> y_data_line(2);

  x_data_line(0) = x;
  x_data_line(1) = x;
  y_data_line(0) = 0;
  y_data_line(1) = chart.yLeftAxis().maxValue();
  time_line.setData(x_data_line, y_data_line);

  drawToBuffer();
  update();
}



void Chart_ECOBB3_Time::drawToBuffer() {
  chart.arrangeDrawing(buffer().width(), buffer().height());
  chart.drawToBuffer(buffer());
}

