
#include "recordview.h"
#include "recorditem.h"
#include "wl/archivefile.h"
#include "wl/record.h"
#include "wl/channel.h"
#include "wl/sensor.h"



RecordView::RecordView(QWidget* p) : 
  QListView(p) { 

  addColumn("Logger");
  addColumn("TP");
  addColumn("Ch");
  addColumn("Sensor");
  addColumn("OK");
  addColumn("Ex.");

  setAllColumnsShowFocus(true);
  setRootIsDecorated(false);

  setColumnAlignment(1, AlignHCenter);
  setColumnAlignment(2, AlignHCenter);
  setColumnAlignment(4, AlignHCenter);
  setColumnAlignment(5, AlignHCenter);
}



// RecordView::populate()

void RecordView::populate(ArchiveFile& af) {

  const LList<Channel>& clist(af.channelList());
  int cnum = clist.count();

  Record* r[cnum];

  for (int i=0;i<cnum;i++) {
    r[i] = clist.at(i)->recordList().first();
  }

  long curr_logger_time = af.loggerStartTime();

  while (true) {

    long min_dt = af.loggerEndTime() - curr_logger_time;
    int next_channel = -1;

    for (int i=0;i<cnum;i++) {

      if (r[i]) {

	if (r[i]->loggerTime() - curr_logger_time <= min_dt) {
	  min_dt = r[i]->loggerTime() - curr_logger_time;
	  next_channel = i;
	}
      }
    }

    if (next_channel < 0) break;

    Record* rec = r[next_channel];

    //printf("%d %s\n", (int)rec->loggerTime(), rec->sensor().typeText());

    if (rec->isComplete()) {

      curr_logger_time = rec->loggerTime();

      double tp_secs = ((double) curr_logger_time - af.loggerStartTime()) / 1000;
      QString tp_str;
      tp_str.sprintf("%.1f",tp_secs);

      RecordItem* item = new RecordItem(*rec, tp_str, next_channel, this);
    }

    r[next_channel] = clist.at(next_channel)->recordList().next();
  }
}


